"use strict";

var _core = require("@syndicate-lang/core");

var _Syndicate = require("@syndicate-lang/core");

var _Dataspace = _Syndicate.Dataspace;
var _Skeleton = _Syndicate.Skeleton;
var _Record = _Syndicate.Record;
var _savedGlobalFacet = _Dataspace._currentFacet;
_Dataspace._currentFacet = new _Syndicate._Dataspace.ActionCollector();

var Http = _Dataspace.activate(require("@syndicate-lang/driver-http-node"));

var S = _Dataspace.activate(require("@syndicate-lang/driver-streams-node"));

var C = require("./compiler");

var J = _Dataspace.activate(require("./job"));

var fs = require('fs');

var options = {
  "port": 14641,
  "babel-options": JSON.parse(fs.readFileSync(__dirname + '/../.babelrc'))
};

function usage() {
  console.info('Usage: syndicate-babel-server [--port HTTPPORT] [--babelrc FILENAME]');
  console.info('                              [--babel-options JSON]');
  console.info(options);
}

function process_command_line(args) {
  while (args.length) {
    switch (args[0]) {
      case '--port':
        {
          args.shift();
          options.port = Number.parseInt(args.shift());
          break;
        }

      case '--babelrc':
        {
          args.shift();
          options["babel-options"] = JSON.parse(fs.readFileSync(args.shift()));
          break;
        }

      case '--babel-options':
        {
          args.shift();
          options["babel-options"] = JSON.parse(args.shift());
          break;
        }

      default:
        console.error("Unsupported command-line argument: " + args[0]);

      /* FALL THROUGH */

      case '--help':
      case '-h':
        usage();
        process.exit(1);
    }
  }
}

process_command_line(process.argv.slice(2));
console.info("export SYNDICATE_COMPILE_SERVER=http://localhost:".concat(options.port, "/compile"));
console.info(options);

_Dataspace.spawn('rootServer', function () {
  _Dataspace._currentFacet.addEndpoint(function () {
    return [C.CompilationOptions(options["babel-options"]), null];
  }, true);

  var server = Http.HttpServer(null, options.port);

  _Dataspace._currentFacet.addEndpoint(function () {
    var _c26 = server;
    var _c27 = 'post';
    var _c28 = 'compile';

    var _assn = _Syndicate.Observe(Http.Request(_Syndicate.Capture(_Syndicate.Discard._instance), _c26, _c27, [_c28, _Syndicate.Capture(_Syndicate.Discard._instance)], _Syndicate.Discard._instance, _Syndicate.Capture(_Syndicate.Discard._instance)));

    var _handler = {
      skeleton: [Http.Request.constructorInfo, null, null, null, [2, null, null], null, null],
      constPaths: _Syndicate.fromJS([_Syndicate.fromJS([1]), _Syndicate.fromJS([2]), _Syndicate.fromJS([3, 0])]),
      constVals: _Syndicate.fromJS([_c26, _c27, _c28]),
      capturePaths: _Syndicate.fromJS([_Syndicate.fromJS([0]), _Syndicate.fromJS([3, 1]), _Syndicate.fromJS([5])]),
      assertion: _assn,
      callback: _Dataspace.wrap(function (_evt, _vs) {
        if (_evt === _Skeleton.EVENT_ADDED) {
          var _reqId = _vs.get(0);

          var _file = _vs.get(1);

          var _reqSeal = _vs.get(2);

          _Dataspace._currentFacet.actor.scheduleScript(function () {
            {
              var _id2 = _Syndicate.genUuid();

              var _inst2 = _Syndicate.Instance(_id2);

              _Dataspace._currentFacet.actor.addFacet(_Dataspace._currentFacet, function () {
                _Dataspace._currentFacet.addEndpoint(function () {
                  var _c29 = _inst2;

                  var _assn2 = _Syndicate.Observe(_c29);

                  var _handler2 = {
                    skeleton: null,
                    constPaths: _Syndicate.fromJS([_Syndicate.fromJS([])]),
                    constVals: _Syndicate.fromJS([_c29]),
                    capturePaths: _Syndicate.fromJS([]),
                    assertion: _assn2,
                    callback: _Dataspace.wrap(function (_evt2, _vs2) {
                      if (_evt2 === _Skeleton.EVENT_ADDED) {
                        _Dataspace._currentFacet.actor.scheduleScript(function () {
                          _Dataspace._currentFacet.stop(function () {
                            _Dataspace._currentFacet.actor.addFacet(_Dataspace._currentFacet, function () {
                              _Dataspace._currentFacet.addEndpoint(function () {
                                var _c30 = _inst2;

                                var _assn3 = _Syndicate.Observe(_c30);

                                var _handler3 = {
                                  skeleton: null,
                                  constPaths: _Syndicate.fromJS([_Syndicate.fromJS([])]),
                                  constVals: _Syndicate.fromJS([_c30]),
                                  capturePaths: _Syndicate.fromJS([]),
                                  assertion: _assn3,
                                  callback: _Dataspace.wrap(function (_evt3, _vs3) {
                                    if (_evt3 === _Skeleton.EVENT_REMOVED) {
                                      _Dataspace._currentFacet.actor.scheduleScript(function () {
                                        _Dataspace._currentFacet.stop(function () {
                                          ;
                                        });
                                      });
                                    }
                                  })
                                };
                                return [_assn3, _handler3];
                              }, true);

                              _Dataspace._currentFacet.addEndpoint(function () {
                                var _c31 = _reqId;
                                var _c32 = server;
                                var _c33 = 'post';
                                var _c34 = ['compile', _file];
                                var _c35 = _reqSeal;

                                var _assn4 = _Syndicate.Observe(Http.Request(_c31, _c32, _c33, _c34, _Syndicate.Discard._instance, _c35));

                                var _handler4 = {
                                  skeleton: [Http.Request.constructorInfo, null, null, null, null, null, null],
                                  constPaths: _Syndicate.fromJS([_Syndicate.fromJS([0]), _Syndicate.fromJS([1]), _Syndicate.fromJS([2]), _Syndicate.fromJS([3]), _Syndicate.fromJS([5])]),
                                  constVals: _Syndicate.fromJS([_c31, _c32, _c33, _c34, _c35]),
                                  capturePaths: _Syndicate.fromJS([]),
                                  assertion: _assn4,
                                  callback: _Dataspace.wrap(function (_evt4, _vs4) {
                                    if (_evt4 === _Skeleton.EVENT_REMOVED) {
                                      _Dataspace._currentFacet.actor.scheduleScript(function () {
                                        _Dataspace._currentFacet.stop(function () {
                                          ;
                                        });
                                      });
                                    }
                                  })
                                };
                                return [_assn4, _handler4];
                              }, false);
                            }, true);
                          });
                        });
                      }
                    })
                  };
                  return [_assn2, _handler2];
                }, true);

                _Dataspace._currentFacet.addEndpoint(function () {
                  var _c36 = _reqId;
                  var _c37 = server;
                  var _c38 = 'post';
                  var _c39 = ['compile', _file];
                  var _c40 = _reqSeal;

                  var _assn5 = _Syndicate.Observe(Http.Request(_c36, _c37, _c38, _c39, _Syndicate.Discard._instance, _c40));

                  var _handler5 = {
                    skeleton: [Http.Request.constructorInfo, null, null, null, null, null, null],
                    constPaths: _Syndicate.fromJS([_Syndicate.fromJS([0]), _Syndicate.fromJS([1]), _Syndicate.fromJS([2]), _Syndicate.fromJS([3]), _Syndicate.fromJS([5])]),
                    constVals: _Syndicate.fromJS([_c36, _c37, _c38, _c39, _c40]),
                    capturePaths: _Syndicate.fromJS([]),
                    assertion: _assn5,
                    callback: _Dataspace.wrap(function (_evt5, _vs5) {
                      if (_evt5 === _Skeleton.EVENT_REMOVED) {
                        _Dataspace._currentFacet.actor.scheduleScript(function () {
                          _Dataspace._currentFacet.stop(function () {
                            _Dataspace._currentFacet.actor.addFacet(_Dataspace._currentFacet, function () {
                              _Dataspace._currentFacet.addEndpoint(function () {
                                var _c41 = _inst2;

                                var _assn6 = _Syndicate.Observe(_c41);

                                var _handler6 = {
                                  skeleton: null,
                                  constPaths: _Syndicate.fromJS([_Syndicate.fromJS([])]),
                                  constVals: _Syndicate.fromJS([_c41]),
                                  capturePaths: _Syndicate.fromJS([]),
                                  assertion: _assn6,
                                  callback: _Dataspace.wrap(function (_evt6, _vs6) {
                                    if (_evt6 === _Skeleton.EVENT_ADDED) {
                                      _Dataspace._currentFacet.actor.scheduleScript(function () {
                                        _Dataspace._currentFacet.stop(function () {
                                          ;
                                        });
                                      });
                                    }
                                  })
                                };
                                return [_assn6, _handler6];
                              }, true);
                            }, true);
                          });
                        });
                      }
                    })
                  };
                  return [_assn5, _handler5];
                }, false);
              }, true);

              _Dataspace.spawn(_reqId, function () {
                _Dataspace._currentFacet.addEndpoint(function () {
                  return [_inst2, null];
                }, true);

                _Dataspace._currentFacet.addEndpoint(function () {
                  var _c42 = _Syndicate.Observe(_inst2);

                  var _assn7 = _Syndicate.Observe(_c42);

                  var _handler7 = {
                    skeleton: null,
                    constPaths: _Syndicate.fromJS([_Syndicate.fromJS([])]),
                    constVals: _Syndicate.fromJS([_c42]),
                    capturePaths: _Syndicate.fromJS([]),
                    assertion: _assn7,
                    callback: _Dataspace.wrap(function (_evt7, _vs7) {
                      if (_evt7 === _Skeleton.EVENT_REMOVED) {
                        _Dataspace._currentFacet.actor.scheduleScript(function () {
                          _Dataspace._currentFacet.stop(function () {
                            ;
                          });
                        });
                      }
                    })
                  };
                  return [_assn7, _handler7];
                }, true);

                {
                  _Dataspace._currentFacet.addEndpoint(function () {
                    var _c43 = S.Stream(_reqId, S.Readable());

                    var _assn8 = _Syndicate.Observe(_c43);

                    var _handler8 = {
                      skeleton: null,
                      constPaths: _Syndicate.fromJS([_Syndicate.fromJS([])]),
                      constVals: _Syndicate.fromJS([_c43]),
                      capturePaths: _Syndicate.fromJS([]),
                      assertion: _assn8,
                      callback: _Dataspace.wrap(function (_evt8, _vs8) {
                        if (_evt8 === _Skeleton.EVENT_REMOVED) {
                          _Dataspace._currentFacet.actor.scheduleScript(function () {
                            _Dataspace._currentFacet.stop(function () {
                              ;
                            });
                          });
                        }
                      })
                    };
                    return [_assn8, _handler8];
                  }, true);

                  _collectSource.call(this, _reqId, function (source) {
                    _Dataspace._currentFacet.actor.addFacet(_Dataspace._currentFacet, function () {
                      var job = C.Compilation(_file, source);

                      _Dataspace._currentFacet.addEndpoint(function () {
                        var _c44 = job;

                        var _assn9 = _Syndicate.Observe(J.Job(_c44, J.JobError(_Syndicate.Capture(_Syndicate.Discard._instance))));

                        var _handler9 = {
                          skeleton: [J.Job.constructorInfo, null, [J.JobError.constructorInfo, null]],
                          constPaths: _Syndicate.fromJS([_Syndicate.fromJS([0])]),
                          constVals: _Syndicate.fromJS([_c44]),
                          capturePaths: _Syndicate.fromJS([_Syndicate.fromJS([1, 0])]),
                          assertion: _assn9,
                          callback: _Dataspace.wrap(function (_evt9, _vs9) {
                            if (_evt9 === _Skeleton.EVENT_ADDED) {
                              var _errmsg = _vs9.get(0);

                              _Dataspace._currentFacet.actor.scheduleScript(function () {
                                _Dataspace._currentFacet.stop(function () {
                                  {
                                    _Dataspace._currentFacet.actor.addFacet(_Dataspace._currentFacet, function () {
                                      _Dataspace._currentFacet.addEndpoint(function () {
                                        return [Http.Response(_reqId, 400, "Error", {
                                          "Content-Type": "text/plain"
                                        }, _errmsg), null];
                                      }, true);
                                    }, true);
                                  }
                                });
                              });
                            }
                          })
                        };
                        return [_assn9, _handler9];
                      }, true);

                      _Dataspace._currentFacet.addEndpoint(function () {
                        var _c45 = job;

                        var _assn10 = _Syndicate.Observe(J.Job(_c45, J.JobResult(_Syndicate.Capture(_Syndicate.Discard._instance))));

                        var _handler10 = {
                          skeleton: [J.Job.constructorInfo, null, [J.JobResult.constructorInfo, null]],
                          constPaths: _Syndicate.fromJS([_Syndicate.fromJS([0])]),
                          constVals: _Syndicate.fromJS([_c45]),
                          capturePaths: _Syndicate.fromJS([_Syndicate.fromJS([1, 0])]),
                          assertion: _assn10,
                          callback: _Dataspace.wrap(function (_evt10, _vs10) {
                            if (_evt10 === _Skeleton.EVENT_ADDED) {
                              var _output = _vs10.get(0);

                              _Dataspace._currentFacet.actor.scheduleScript(function () {
                                _Dataspace._currentFacet.stop(function () {
                                  {
                                    _Dataspace._currentFacet.actor.addFacet(_Dataspace._currentFacet, function () {
                                      _Dataspace._currentFacet.addEndpoint(function () {
                                        return [Http.Response(_reqId, 200, "OK", {}, _output), null];
                                      }, true);
                                    }, true);
                                  }
                                });
                              });
                            }
                          })
                        };
                        return [_assn10, _handler10];
                      }, true);
                    }, true);
                  });
                }
              }, _Syndicate.Set([_inst2, _Syndicate.Observe(_Syndicate.Observe(_inst2))]));
            }
          });
        }
      })
    };
    return [_assn, _handler];
  }, true);
});

function _collectSource(streamId, cb) {
  var chunks = [];

  _Dataspace._currentFacet.addEndpoint(function () {
    var _c46 = streamId;

    var _assn11 = _Syndicate.Observe(S.Stream(_c46, S.Data(_Syndicate.Capture(_Syndicate.Discard._instance))));

    var _handler11 = {
      skeleton: [S.Stream.constructorInfo, null, [S.Data.constructorInfo, null]],
      constPaths: _Syndicate.fromJS([_Syndicate.fromJS([0])]),
      constVals: _Syndicate.fromJS([_c46]),
      capturePaths: _Syndicate.fromJS([_Syndicate.fromJS([1, 0])]),
      assertion: _assn11,
      callback: _Dataspace.wrap(function (_evt11, _vs11) {
        if (_evt11 === _Skeleton.EVENT_MESSAGE) {
          var _chunk = _vs11.get(0);

          _Dataspace._currentFacet.actor.scheduleScript(function () {
            chunks.push(_chunk);
          });
        }
      })
    };
    return [_assn11, _handler11];
  }, true);

  _Dataspace._currentFacet.addEndpoint(function () {
    var _c47 = S.Stream(streamId, S.End());

    var _assn12 = _Syndicate.Observe(_c47);

    var _handler12 = {
      skeleton: null,
      constPaths: _Syndicate.fromJS([_Syndicate.fromJS([])]),
      constVals: _Syndicate.fromJS([_c47]),
      capturePaths: _Syndicate.fromJS([]),
      assertion: _assn12,
      callback: _Dataspace.wrap(function (_evt12, _vs12) {
        if (_evt12 === _Skeleton.EVENT_ADDED) {
          _Dataspace._currentFacet.actor.scheduleScript(function () {
            {
              var source = _core.Bytes.concat(chunks).fromUtf8();

              cb(source);
            }
          });
        }
      })
    };
    return [_assn12, _handler12];
  }, true);
}

{
  var nCPUs = require('os').cpus().length;

  for (var i = 0; i < nCPUs; i++) {
    // spawn dataspace activate C;
    (0, _core.spawnWorker)(__dirname + '/compiler.js');
  }
}
module.exports[_Dataspace.BootSteps] = {
  module: module,
  steps: _Dataspace._currentFacet.actions
};
_Dataspace._currentFacet = _savedGlobalFacet;
_savedGlobalFacet = null;

if ((typeof require === 'undefined' ? {
  main: void 0
} : require).main === module) {
  _Syndicate.bootModule(module);
}

